const path = require('path')
const webpack = require('webpack')
const createThemeColorReplacerPlugin = require('./config/plugin.config')
const HardSourceWebpackPlugin = require('hard-source-webpack-plugin')

// const Happypack = require('happypack')
const { name } = require('./package')
function resolve(dir) {
  return path.join(__dirname, dir)
}

// vue.config.js
const vueConfig = {
  configureWebpack: {
    // externals: prodExternals,
    plugins: [
      // Ignore all locale files of moment.js
      new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
      new HardSourceWebpackPlugin(),
      new HardSourceWebpackPlugin.ExcludeModulePlugin([
        {
          test: /mini-css-extract-plugin[\\/]dist[\\/]loader/
        }
      ])
    ],
    output: {
      // 把子应用打包成 umd 库格式
      library: `${name}-[name]`,
      libraryTarget: 'umd',
      jsonpFunction: `webpackJsonp_${name}`
    }
  },

  chainWebpack: config => {
    config.plugins.delete('preload') // 移除preload
    config.plugins.delete('prefetch') // prefetch
    config.resolve.alias.set('@$', resolve('src'))

    const svgRule = config.module.rule('svg')
    svgRule.uses.clear()
    svgRule
      .oneOf('inline')
      .resourceQuery(/inline/)
      .use('vue-svg-icon-loader')
      .loader('vue-svg-icon-loader')
      .end()
      .end()
      .oneOf('external')
      .use('file-loader')
      .loader('file-loader')
      .options({
        name: 'assets/[name].[hash:8].[ext]'
      })
    config.module
      .rule('images')
      .use('url-loader')
      .loader('url-loader')
      .options({
        limit: 16384, // 小于4kb将会被打包成 base64
        fallback: {
          loader: 'file-loader',
          options: {
            name: 'img/[name].[hash:8].[ext]',
            publicPath: process.env.VUE_APP_PUBLIC_PATH
          }
        }
      })
    // assets require on cdn
    // config.plugin('html').tap(args => {
    //   args[0].cdn = assetsCDN
    //   return args
    // })
    config.when(process.env.NODE_ENV !== 'development', config => {
      // runtime文件内联，减少网络请求耗时
      config
        .plugin('ScriptExtHtmlWebpackPlugin')
        .after('html')
        .use('script-ext-html-webpack-plugin', [
          {
            inline: /runtime\..*\.js$/
          }
        ])
        .end()
      config.optimization.splitChunks({
        chunks: 'all',
        minSize: 20000, // 字节 引入的文件大于20kb才进行分割
        minChunks: 1, // 模块至少使用次数
        cacheGroups: {
          libs: {
            name: 'chunk-libs',
            test: /[\\/]node_modules[\\/]/,
            priority: 10,
            chunks: 'initial' // only package third parties that are initially dependent
          },
          ui: {
            name: 'chunk-ui',
            priority: 20,
            test: /[\\/]node_modules[\\/]_?ant-design-vue(.*)/
          },
          lodash: {
            name: 'chunk-lodash',
            priority: 20,
            test: /[\\/]node_modules[\\/]_?lodash(.*)/
          },
          tinymce: {
            name: 'chunk-tinymce',
            priority: 20,
            test: /[\\/]node_modules[\\/]_?tinymce(.*)/
          },
          quill: {
            name: 'chunk-quill',
            priority: 20,
            test: /[\\/]node_modules[\\/]_?quill(.*)/
          },
          moment: {
            name: 'chunk-moment',
            priority: 20,
            test: /[\\/]node_modules[\\/]_?moment(.*)/
          },
          commons: {
            name: 'chunk-commons',
            test: resolve('src/components'), // can customize your rules
            minChunks: 3, //  minimum common number
            priority: 5,
            reuseExistingChunk: true
          }
        }
      })
      config.optimization.runtimeChunk('single')
    })
  },

  css: {
    loaderOptions: {
      less: {
        modifyVars: {
          // less vars，customize ant design theme
          'primary-color': '#3377FF',
          // 'link-color': '#F5222D',
          'border-radius-base': '4px'
          // 'border-radius-base': '2px'
        },
        javascriptEnabled: true
      }
    }
  },

  devServer: {
    // development server port 8000
    port: 8003,
    proxy: {
      '/api': {
        target: `http://112.5.142.51:14021/api`,
        pathRewrite: { '^/api': '' },
        ws: false,
        changeOrigin: true,
        hot: true
      },
      '/fileApi': {
        target: `http://112.5.142.51:14021/fileApi`,
        pathRewrite: { '^/fileApi': '' },
        ws: false,
        changeOrigin: true
      }
    },
    headers: {
      'Access-Control-Allow-Origin': '*'
    },
    publicPath: `/`
  },
  outputDir: `dist`,
  publicPath: process.env.VUE_APP_PUBLIC_PATH,
  // disable source map in production；
  productionSourceMap: false,
  // lintOnSave: undefined,
  lintOnSave: false,
  // babel-loader no-ignore node_modules/*
  transpileDependencies: []
}

// 如果你不想在生产环境开启换肤功能，请打开下面注释

if (process.env.VUE_APP_PREVIEW === 'true') {
  // add `ThemeColorReplacer` plugin to webpack plugins
  vueConfig.configureWebpack.plugins.push(createThemeColorReplacerPlugin())
}

module.exports = vueConfig
